// classes for communication with UT2XXX

/*
#########################################################
#   Program pro ovladani osciloskopu UT2XXX             #
#                                                       #
#   Ing. Košan Tomáš, prosinec 2008                     #
#   Peeter Vois peeter@tauria.ee 2012                   #
#                                                       #
#   Copyright (C) 2012 Peeter Vois                      #
#   Licence: GNU GPL v2 or any later version            #
#########################################################
*/



#ifndef __UT2XXX_h__
#define __UT2XXX_h__

#include "ut2XXX_definitions.h"
#include <usb.h>
#include <map>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
using namespace std;

	typedef struct
	{
		bool		far_mode;
		bool		locked;
		char		info_data[65];
	}t_status;

	typedef struct
	{
		unsigned char	samples[5000];
		double		V_div;
		double		s_div;
		char*		couple;
		bool		active;
		int		x_offset;		
		bool		BW_limit;
		unsigned char	header[100];
		int		probe;
		int		probe_index;
		bool		changed;
		int		V_div_index;
		int		s_div_index;
		int		couple_index;
		bool		inverted;
		int		y_offset;
		int		y_pos;
		int		x_pos;
	} t_ch_data;


class UNI_TREND_DSO
{
private:
	t_status 	status;		// init status
	t_ch_data 	ch_data[2];	// channel data

public:
	UNI_TREND_DSO( int E_VID, int E_PID )
	{
	    usb_init();
	    usb_find_busses();
	    usb_find_devices();
		busses = usb_get_busses();
		//status = (t_status){false,false,""};
		//ch_data[0] = (t_ch_data){"",0,0,"",false,0,false,"",0,0,false,0,0,0,false,125,3803,0};
		//ch_data[1] = ch_data[0];
		error_message = "";
		VID[ 0x5656 ] = 0x5656; PID[ 0x0834 ] = 0x0834;
		if( E_VID ) VID[E_VID] = E_VID;
		if( E_PID ) PID[E_PID] = E_PID;
		init();
		//printf( "init_done\n" );
	};
	~UNI_TREND_DSO()
	{
		close();
	}
	void test()
	{
		if( device == 0 ) printf( "No device found!\n" );
		int n = get_waveform();
		if( n== 0 ) printf( "No data received!\n" );
		for( int i=0; i<n; i++ )
		{
			if( i % 25 == 0 ) printf("\n%03d",i);
			if( i % 5 == 0 ) printf(" ");
			printf( "%02hhx", ch_data[0].samples[i] );
		}		
		printf( "\n" );
	};
	int get( double *buf, int rows, int cols=3 )
	{
		int n = get_waveform();
		for( int i=0; (i<n) && (i<rows); i++ )
		{
			double y[2];
			for( int c=0; c<2; c++ )
			{
				y[c] = (double)ch_data[c].samples[i] -254.0 + (double)ch_data[c].header[Y_POS];
				y[c] *= Y_RANGE[ch_data[c].header[Y_SENSE]] *0.04;
				y[c] *= PROBE[ ch_data[c].header[Y_PROBE] ];
			}
			double x = (double)i;
			x -= (double)ch_data[0].header[X_CURSOR];
			x *= X_RANGE[ ch_data[0].header[X_SCALE] ];
			x *= 0.04;
			if( cols > 1 ) *(buf+i*3+0) = x; // make it scalable time
			*(buf+i*3+1) = y[0];
			if( cols > 2 ) *(buf+i*3+2) = y[1];
		}
		return( n );
	};
private:
	//vid and pid
	map<int,int>		VID;
	map<int,int>		PID;
	//init status
	//p	self.data_old = ()
	//p	self.pixmap_data = ()
	char			*error_message;
	struct usb_device	*device;
	usb_dev_handle		*handle;
	struct usb_bus 		*busses;
	int			interface_id;
	bool			is_present;
	char			product[51];
	char			manufacturer[51];
	char			data[10000];
	// leaves far mode - unblocks local control of osciloscope
	bool leave_far_mode()
	{
		int r = usb_control_msg( handle, 0x42, 0xb1, 0xf1, 0, 0, 0, 200 );
		//printf( "r1=%d\n", r );
		//handle.controlMsg(0x42,0xb1,None,value=0xf1);
		// failure handling
		status.far_mode = false;
		return( true );		
	};
	//
	// initialize
	//
	void init_device()
	{
		int r;
		//char ret[100]="";
		//int r = usb_control_msg( handle, 0xc2, 0xb2, 0x08, 4, ret, 8, 300 );
		//printf( "r=%d\n", r );
		//for( int i=0; i< r; i++ ) printf( "%02hhx:%c,", ret[i], ret[i] ); printf( "\n" );
		//printf( "%s\n", handle.controlMsg(0xc2,0xb2,8,index=4, value=0x08, timeout=300)	);
		r = usb_control_msg( handle, 0x42, 0xb1, 0x2c, 0, 0, 0, 300 );
		//handle.controlMsg(0x42,0xb1,None,index=0, value=0x2c);
		usleep( 30000 );
		//time.sleep(0.02)	
		r = usb_control_msg( handle, 0x42, 0xb1, 0xdc, 0, 0, 0, 300 );
		//printf( "r1=%d\n", r );
		//handle.controlMsg(0x42,0xb1,None,index=0, value=0xdc);
		usleep( 30000 );
		//time.sleep(0.02)	
		r = usb_control_msg( handle, 0x42, 0xb1, 0xcc, 0, 0, 0, 300 );
		//printf( "r1=%d\n", r );
		//handle.controlMsg(0x42,0xb1,None,index=0, value=0xcc);
		usleep( 30000 );
		//time.sleep(0.02)	
		r = usb_control_msg( handle, 0x42, 0xb1, 0xcc, 0, 0, 0, 300 );
		//printf( "r1=%d\n", r );
		//handle.controlMsg(0x42,0xb1,None,index=0, value=0xcc);
		usleep( 30000 );
		//time.sleep(0.02)
		leave_far_mode();
	};
	//
	void init()
	{
		device = 0;
		handle = 0;
		interface_id = 0;
		find_device();
		if( device == 0 )
		{
			//printf("Device not found.");
			is_present = false;
		} 
		else 
		{
			is_present = true;
			handle = usb_open( device );
			if( handle == 0 ) printf( "usb_open failed!\n" );
			//usb_get_string_simple( handle, 2, product, 50 ); printf( " Product: %s\n", product );
			//usb_get_string_simple( handle, 1, manufacturer, 50 );  printf( " Manufacturer: %s\n", manufacturer );
			int r = usb_claim_interface( handle, interface_id );
			if( r < 0 ) printf( "Claim interface failed: %d\n", r );
			init_device();
		}
		//data = "";
	};	
	// try to find proper USB device and get her config
	void find_device()
	{
	struct usb_bus *bus;
		device = 0;
		for( bus = busses; bus && (device == 0); bus = bus->next )
		{
		struct usb_device *dev;
		 for( dev = bus->devices; dev && (device == 0); dev = dev->next )
		 {
		  if( PID[ dev->descriptor.idProduct ] == dev->descriptor.idProduct &&
		      VID[ dev->descriptor.idVendor ] == dev->descriptor.idVendor )
		  {
		   device = dev;
		  }
		 }
		}
	};
  	// enters far mode - it means all control on osciloscope is blocked
	bool enter_far_mode()
	{
		int r = usb_control_msg( handle, 0x42, 0xb1, 0xf0, 0, 0, 0, 300 ); 
		//printf( "r1=%d\n", r );
		//handle.controlMsg(0x42,0xb1,None,value=0xf0);
		// failure handling
		status.far_mode = true;
		return( true );
	};
	//
	bool get_info_from_device()
	{
		// asi nastaveni jednotlivych ovl. prvku
		// maybe state of control
		usb_control_msg( handle, 0x42, 0xb1, 0x2c, 0, 0, 0, 300 );
		//handle.controlMsg(0x42,0xb1,None, value=0x2c);
		usb_control_msg( handle, 0x00, 0xb2, 0, 0, 0, 0, 300 );
		//handle.controlMsg(0x00,0xb2,None);
		usb_control_msg( handle, 0xc2, 0xb2, 8, 0, status.info_data, 64, 300 );
		//status.info_data = handle.controlMsg(0xc2,0xb2, 8);
		return( true );
	};
	//
	int get_data(char *data)
	{
		if( is_present == false ) return( 0 );
		int i,n=0;
		for( i=0; (i<10) && (n==0); i++ )
		{
			enter_far_mode();
			int x = usb_control_msg( handle, 0x42, 0xb1, 0xe1, 0, 0, 0, 300 );
			//printf( "x1=%d\n", x );
			//handle.controlMsg(0x42,0xb1,None,index=0, value=0xe1);
			usleep( 20000 );
			//time.sleep(0.03)	
			x = usb_control_msg( handle, 0x42, 0xb0, 0x01, 2, 0, 0, 300 );
			//printf( "x1=%d\n", x );
			//handle.controlMsg(0x42,0xb0,None,index=2, value=0x01);	
			usleep( 20000 );
			//#time.sleep(0.02)	
			int r = usb_bulk_read( handle, 0x82, data+n , 1024, 300 );
			//data = handle.bulkRead(130,1024,60);
			usleep( 20000 );
			//#time.sleep(0.01)
			x = usb_control_msg( handle, 0x42, 0xb0, 0xdc, 0, 0, 0, 300 );		
			//printf( "x1=%d\n", x );
			//self.handle.controlMsg(0x42,0xb0,None,index=0, value=0xdc);
			//if( r <=0 ){ printf( "bulk read error:%d\n",r); break; }
			if( r > 0 ) n += r;
			leave_far_mode();
			if(r <= 0 )
				usleep( 400000 );
		}
		//printf( "data read %d bytes\n", n );
		//for( i=0; i<n; i++ )
		//{
		//	if( i % 16 == 0 ) printf("\n%04d",i);
		//	if( i % 4 == 0 ) printf(" ");
		//	printf( "%02hhx", data[i] );
		//}
		return( n );
	};
	//
	int get_parameters(char *data)
	{
		int n=0;
		usb_control_msg( handle, 0x42, 0xb1, 0xe3, 0, 0, 0, 200 );
		//handle.controlMsg(0x42,0xb1,None,index=0, value=0xe3);
		//time.sleep(0.05)
		usb_control_msg( handle, 0x42, 0xb0, 0, 0, 0, 0, 200 );	
		//handle.controlMsg(0x42,0xb0,None);
		n += usb_bulk_read( handle, 0x82, data, 512, 500 );
		//data = handle.bulkRead(0x82,512,500);
		usb_control_msg( handle, 0x42, 0xb0, 0xdc, 0, 0, 0, 200 );
		//handle.controlMsg(0x42,0xb0,None,index=0, value=0xdc);
		return( n );
	};
	//
	void ping()
	{
		usb_control_msg( handle, 0x42, 0xb0, 0xdc, 0, 0, 0, 200 );
		//handle.controlMsg(0x42,0xb0,None,index=0, value=0xdc);
		//time.sleep(0.02)	
		usb_control_msg( handle, 0x42, 0xb1, 0xcc, 0, 0, 0, 200 );
		//handle.controlMsg(0x42,0xb1,None,index=0, value=0xcc);	
		//time.sleep(0.02)	
	};
	//
	void close()
	{
		if( is_present == false ) 
			return;
		//printf( "Closing, bye.\n" );
		leave_far_mode();
		usb_release_interface( handle, interface_id );
		//handle.releaseInterface();
	};
	// get samples from DSO
	int get_waveform()
	{
		int datalen = get_data( data );

		if( datalen <= 0 )
		{
			//printf( "No data received: %d\n", datalen );
			return(0);
		}
		
		//# check for proper length of data packet	
		//if not data == None and len(data) >= 1024:
			
			for( int c=0; c<2; c++ )
			{
				if( memcmp( ch_data[c].header, data+c*32, 32 ) != 0 )
					ch_data[c].changed = true;
				else
					ch_data[c].changed = false;
				
				memcpy( ch_data[c].header, data+c*32, 32 );
			
				// compute V/div for each chanel
				ch_data[c].V_div = Y_RANGE[ch_data[c].header[Y_SENSE]];
					//*(10*(ch_data[c].header[Y_PROBE_CH1]));
				ch_data[c].V_div_index = ch_data[c].header[Y_SENSE];
				// probe
				ch_data[c].probe = 10*(ch_data[c].header[Y_PROBE]);
				ch_data[c].probe_index = ch_data[c].header[Y_PROBE];
				// check for coupling
				ch_data[c].couple = COUPLING_CHR[ch_data[c].header[COUPLING]];
				ch_data[c].couple_index = ch_data[c].header[COUPLING];
				// save samples data to buffers
				if( datalen == 1024 )
					memcpy( ch_data[c].samples, data+516+c*254, 250 );
				if( datalen == 2560 )
					memcpy( ch_data[c].samples, data+516+c*1004, 750 );
				
				// compute s/div
				ch_data[c].s_div = X_RANGE[ch_data[c].header[X_SCALE]];
				ch_data[c].s_div_index = ch_data[c].header[X_SCALE];
				// check status of channels
				ch_data[c].active = bool(data[CHANNEL_STATE] & (0x01+c));
				// x offset
				ch_data[c].y_offset = 0x7e - ch_data[c].header[Y_POS];
			
				ch_data[c].BW_limit = bool(ch_data[c].header[BW_LIMIT]);
			
				ch_data[c].inverted = bool(ch_data[c].header[INVERTED]);
				
				ch_data[c].x_offset = ch_data[c].header[X_CURSOR];
			
				//? ch_data[c].x_pos = (data[8+c*32] << 8) + data[7+c*32];
			}
		if( datalen == 1024 ) return( 250 );
		if( datalen == 2560 ) return( 750 );
		return( 0 );
	};
};

#endif // __UT2XXX_h__

