// Definitions of the UT2XXX

/*
####################################################################
#   Definice komunikacnich prikazu pro osciloskop UT2062C          #
#   Definitions of commands for communication with DSO             #
#                                                                  #
#   Ing. Košan Tomáš, prosinec 2008                                #
#   Peeter Vois peeter@tauria.ee 2012                              #
#                                                                  #
#   Copyright (C) 2012 Peeter Vois                                 #
#   Licence: GNU GPL v2 or any later version                       #
####################################################################
*/


#ifndef __UT2XXX_definitions_h__
#define __UT2XXX_definitions_h__

#define X_OFFSET_NULL	3803

// prikazy pro vzdalene ovladani
// remote control commands

// switch on/off remote control
// zapne/vypne vzdalene ovladani
#define REMOTE_ON	0xf0
#define REMOTE_OFF	0xf1

// F1 - F5 buttons
// tlacitka F1 - F5
#define BTN_F1		0xf7
#define BTN_F2		0xf6
#define BTN_F3		0x03
#define BTN_F4		0x04
#define BTN_F5		0x05

// display control buttons
// tlacitka pro zobrazovani
#define BTN_CH1		0x07
#define BTN_CH2		0x09
#define BTN_MATH	0x0a
#define BTN_REF		0x0b
#define BTN_OFF		0x18

// funtion buttons
// tlacitka funkci
#define BTN_MEAS	0x16
#define BTN_ACQU	0x1c
#define BTN_STOR	0x22
#define BTN_RUN_STOP	0x28
#define BTN_CURSOR	0x17
#define BTN_DISPLAY	0x1d
#define BTN_UTILITY	0x23
#define BTN_AUTO	0x29
#define BTN_MENU	0x24

// X axis control
// ovladani X osy
#define VOLTS_UP	0x36
#define VOLTS_DN	0x35
#define VERT_UP		0x33
#define VERT_DN		0x34
#define SET_TO_0	0x0c

// Y axis control
// ovladani Y osy
#define SEC_DIV_UP	0x39
#define SEC_DIV_DN	0x3a
#define HORIZ_UP	0x38
#define HORIZ_DN	0x37

// trigger control
// ovladani trigeru
#define TRIG_UP		0x3b
#define TRIG_DN		0x3c
#define MENU_TRIG	0x2e
#define SET_TO_50	0x2f
#define FORCE_TRIG	0x30

// univerzal turnning part
// univerzalni tocitko
#define UNI_PLUS	0x31
#define UNI_MINUS	0x32

// help
// napoveda
#define BTN_HELP	0x2a

// data load
// nacitani dat
#define DATA_CH1	0xf9
#define DATA_CH2	0xfa
#define GET_WAVE	0x06

// ranges
// X axis range in V/div
#define Y_RANGE	\
	(double[]){2E-3, 5E-3, 10E-3, 20E-3, 50E-3, 100E-3, 200E-3, 500E-3, 1, 2, 5}

#define PROBE \
	(double[]){ 1E0, 10E0, 100E0, 1E3, 10E3 }

#define X_RANGE	\
	(double[]){0, 2E-9, 5E-9, 10E-9, 20E-9, 50E-9, 100E-9, 200E-9, 500E-9, 1E-6, 2E-6,\
	5E-6, 10E-6, 20E-6, 50E-6, 100E-6, 200E-6, 500E-6, 1E-3, 2E-3, 5E-3, 10E-3,\
	20E-3, 50E-3, 100E-3, 200E-3, 500E-3, 1, 2, 5, 10, 20, 50}
	
#define COUPLING_CHR \
	(char*[]){"DC","AC","GND"}
	
// indexes of data sections
#define CHANNEL_STATE	2

#define CH_OFFSET		32

#define Y_SENSE			5

#define Y_POS			6

#define Y_PROBE			19

#define COUPLING		12

#define X_SCALE			10

#define X_CURSOR		11

#define BW_LIMIT		15

#define INVERTED		9

#endif //__UT2XXX_definitions_h__

